REBOL [
	Title: "Associate REBOL .r file in Windows"
	Rights: "Copyright REBOL Technologies 1997-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

set-action: func [key from name /local value new-key] [
	new-key: join key "\command"
	any [
		get-reg/hkcr new-key ""
		all [
			value: get-reg/hkcr from ""
			create-reg/hkcr new-key
			set-reg/hkcr new-key "" value
			set-reg/hkcr key "" name
		]
	]
]

associate-file: func [
	"Set file association for REBOL on Win32"
	product [string!] "Product name"
	rebol-path [file!] "Path to REBOL exe"
	/local key new-key value suffix create-hkcr set-hkcr get-hkcr debug
][
	; Helpers (to remove path usage below)
	create-hkcr: func [key] [create-reg/hkcr key]
	set-hkcr: func [key id value] [set-reg/hkcr key id value]
	get-hkcr: func [key id] [get-reg/hkcr key id]
	debug: func [msg] [] ;print ["Associate:" msg]]

	rebol-path: to-local-file rebol-path

	; Create the .r file association record.
	; Set content type to the REBOL MIME type.
	; Create a shellnew value with a nullfile attribute. (Why?)
	debug ".r key"
	create-hkcr ".r"
	key: get-hkcr ".r" ""
	set-hkcr ".r" "Content Type" "text/x-rebol"
	create-hkcr ".r\ShellNew"
	set-hkcr ".r\ShellNew" "NullFile" ""
	if any [not key empty? key] [key: "REBOL.Script"]
	set-hkcr ".r" "" key

	; Return TRUE from function only if we succeed:
	all [
		; Create the rebol.script record and set the TYPE string and icon:
		debug "rebol.script key"
		create-hkcr key
		create-hkcr join key "\DefaultIcon"
		set-hkcr key "" "REBOL Script"
		set-hkcr key "EditFlags" #{00000000}
		set-hkcr join key "\DefaultIcon" "" join rebol-path ",0"

		; Create record for the execution command string.
		; Each REBOL product uses a different command path to allow
		; different types of REBOL on the same machine.
		debug "shell key"
		new-key: join key ["\shell\" product "\command"]
		create-hkcr new-key
		set-hkcr new-key "" rejoin [{"} rebol-path {" -q "%1"}]
		set-hkcr join key "\shell" "" product ; default action?
		set-hkcr join key ["\shell\" product] "" join "&" product

		; Create EDIT action that allows a default text editor to
		; open REBOL script files.
		debug "edit key"
		set-action join key "\shell\Edit" "txtfile\shell\open\command" "&Edit"
		; Necessary for shellnew action (new REBOL script) to work:
		set-action join key "\shell\Open" join key "\shell\Edit\command" "&Open"
		true
	]
]

unassociate-file: has [key][
	; Remove REBOL .r associations.
	key: get-reg/hkcr ".r" ""
	if key [
		delete-reg-hkcr key
		delete-reg-hkcr ".r"
	]
]

associated-file?: has [key][
	; If associated, returns REBOL.script.
	get-reg/hkcr ".r" ""
]

;do %utils.r
;print associated-file?
;unassociate-file
;print associate-file "View" "f:\build\to-win32\viewpro\rebol.exe"

