REBOL [
	Title: "REBOL/View Desktop - Network Connections"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

connecting: none  ; in the process of connecting

connect-view: has [info] [
	; Connect the desktop to the Internet and check the rebol.com site for updates.
	if connecting [connecting: false exit]
	connecting: true
	show-status "connecting"
	show-progress/no-status 100 1
	dbug ["Connected?" connected?]
	either all [
		connected?
		any [find [socks socks5 generic] system/schemes/default/proxy/type find-site "www.rebol.com"]
		info: read-thru/progress/update/expand http://www.rebol.com/view/update.r :show-progress
	][
		dbug "Online"
		show-progress 1 1
		alive?: true
		clear index-cache
		if error? try [do as-string info][
			alert "Error on connection. Please report this on www.REBOL.com. Thank you."
		]
		show-main-status reform ["Connected to the Internet at" now/time]
		show-status ""
	][
		dbug "Offline"
		show-progress 1 1
		show-main-status {Cannot connect. Working offline.}
		tell {Click on "Local" to connect.}
		show-status "Local"
		alive?: false
	]
]

find-site: func [host [string!] /local ip dport n secs] [
	; Find IP address of the specified host by doing an async DNS lookup.
	; Do not return until the request has completed or an error happened.
	; GUI events will still be processed (because WAIT is called), but this
	; is NOT the central call to the WAIT function. Avoid nesting WAIT.
	; This is an old way of doing an asynchronous DNS lookup.
	if error? try [dport: open/no-wait dns:///async][
		return read to-url rejoin ["dns://" host] ; fallback for some machines
	]
	insert dport host
	n: 0
	secs: any [user-prefs/connect-wait 60]
	while [error? try [ip: copy dport]] [
		wait [dport 1]
		n: n + 1
		show-progress/no-status secs n
		if any [not connecting n >= secs] [ip: none break]
	]
	connecting: false
	close dport
	ip
]


