REBOL [
	Title: "REBOL/View Desktop - Folders"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

this-path: none		; path to current folder
folder-stack: []	; backtracking folders

show-folder: func [folder /local path] [
	; Given a folder path, load the folder's index file, load its
	; preference colors, image, and effects, and layout its icons.
	if none? folder [alert "Folder has no path." exit]
	path: folder
	if any [url? path file? path] [
		path: full-path path
		if (last path) = #"/" [path: path/index.r]
		either folder: load-index path [this-path: path][
			alert reform ["Cannot open folder:" path] exit
		]
		if path <> pick folder-stack 1 [insert folder-stack path]
		show-path any [folder/title path]
	]
	clear dtc-face/pane
	dtg-path/font/color: any [folder/text-color black]
	dtg-path/font/colors: reduce [dtg-path/font/color any [folder/over-color complement dtg-path/font/color]]

	dtf-face/user-data: reduce ['color folder/color 'image folder/image 'effect any [folder/effect copy []]]
	dtf-face/color: none
	if all [dtf-face/user-data/effect not find dtf-face/user-data/effect 'fit] [append dtf-face/user-data/effect 'fit]	
	if any [dtf-face/user-data/color dtf-face/user-data/image dtf-face/user-data/effect][
		dtw-mid/image: make-bkg dtf-face/size
	]

	if folder/icons [
		foreach icon folder/icons [append dtc-face/pane make-icon icon]
	]
	layout-icons dtc-face/pane
	show-icons ; dtf-face
]

back-folder: does [
	; Go back to previous folder:
	remove folder-stack
	this-path: pick folder-stack 1
	either this-path [show-folder this-path][show-welcome]
]

show-bookmarks: func [folder /local path xy] [
	; Load a bookmark index file, background image, and effect,
	; then layout its icons and show it.
	clear dtw-sid/pane
	path: full-path folder
	if not folder: load-index path [
		alert reform ["Cannot open bookmarks:" path] exit
	]
	dtw-sid/color: folder/color
	if folder/image [dtw-sid/image: load-safe-image folder/image]
	if folder/effect [dtw-sid/effect: any [folder/effect copy []]]
	if not find dtw-sid/effect 'fit [append dtw-sid/effect 'fit]
	if folder/icons [
		xy: 18x14
		foreach icon folder/icons [
			append dtw-sid/pane icon: make-icon icon
			if not word? icon/user-data/item [
				icon/user-data/item: full-path icon/user-data/item
			]
			icon/offset: xy
			xy: icon/size * 0x1 + (icon-space * 0x1) + xy
		]
	]
	show dtw-sid
]

show-services: func [file /local path services f x] [
	; Load and show the services menu:
	clear next dts-face/pane
	x: dts-face/pane/1/size/x 
	path: full-path file
	if all [services: load-index path  services/icons] [
		foreach icon services/icons [
			if icon/type = 'service [
				f: make-face dts-proto
				f/text: icon/name
				f/size: 200x20
				f/line-list: none
				f/action: compose/deep [
					do-file (either word? icon/item [to-lit-word icon/item][icon/item])
					(if icon/stats [reduce [icon/stats]])
				]
				f/size/x: 4 + first size-text f
				x: x + 8
				f/offset/x: x
				x: x + f/size/x
				append dts-face/pane f
			]
		]
	]
	dtm-services/pane: dts-face/pane
	dtm-services/size: dts-face/size
	dtm-services/size/x: x
	show dtm-services
]

full-path: func [file] [
	; Convert a file or url to a full path name:
	if not any [file? file url? file] [return none]
	if any [url? file  #"/" = pick file 1] [return file]
	if none? this-path [return view-root/:file]
	join first split-path this-path file
]

reload-folder: does [
	; Reload the current folder and show it:
	if this-path [
		if read-binary this-path none [show-folder this-path]
	]
]

explore-dir: func [path] [
	if path [path: attempt [first split-path path-thru full-path path]]
	if not path [path: view-root]
	if file? path [
		either system/version/4 = 3 [
			call rejoin [{%SystemRoot%\Explorer.exe "} to-local-file path {"}]
		][browse path]
	]
]

show-folder-options: has [r] [
	r: request ["Pick a folder option:" "Reload" "Explore" "Cancel"]
	if none? r [exit]
	either r [reload-folder][explore-dir this-path]
]
