REBOL [
	Title: "REBOL/View Desktop - Goto Window"
	Version: 1.0.0
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1999-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

goto-url: func [url [url!] /local page script] [
	; Go to a specified URL, try to read it and check to see if it
	; is a valid folder (index file). If it is, show it, otherwise
	; run it.
	url: copy url
	if script: find/match url "file:" [
		if find/match script "//" [remove script] ; remove first one
		script: to-file script
		if attempt [script? read script] [launch script] [browse script]
		exit
	]
	if not page: attempt [read-binary url none] [
		if %.r <> suffix? url [url: append dirize url "index.r"]
		page: read-binary url none
	]
	either page [
		page: to-string page
		either script? page [
			either not error? try [script: load/all page][
				either index-file? script [show-folder url][launch-thru url]
			][
				alert reform ["Invalid REBOL URL:" url]
			]
		][
			browse url
		]
	][
		alert reform ["Cannot find:" url]
	]
]

goto-view: has [gt-url gt-list gtface h enter-url urls result][
	; Popup a requestor to ask for a URL to goto.
	if error? try [urls: load history-file][urls: []]
	gt-face: center-face layout [
		origin 8
		backeffect base-effect
		space 6x4
		h4 bold "Goto a URL (REBOL, Web, or local file):"
		across
		gt-url: field 300 [enter-url value]
		btn-enter 80 "Goto" [if result: enter-url gt-url/text [unview goto-url result]]
		return
		gt-list: text-list 300x200 data urls [enter-url value]
		h: at
		below at h
		guide
		pad 98
		btn 80 "Find..." [if value: request-file/only/keep [enter-url join file:/ value]]
		btn 80 "Sort" [sort gt-list/data show gt-list]
		btn 80 "Clear All" [attempt [delete history-file] unview]
		btn-cancel 80 "Close" escape [unview]
	]

	enter-url: func [url /local val] [
		url: copy url
		if not url? url [
			if error? try [val: load/all url  val: val/1][val: none]
			if not url? :val [
				insert url http://
				if error? try [val: load/all url val: val/1][val: none]
			]
			if not url? :val [alert "Invalid URL." return none]
			url: val
		]
		insert clear gt-url/text url
		remove find gt-list/data url
		insert gt-list/data url
		clear gt-list/picked
		show [gt-url gt-list]
		save history-file gt-list/data
		url
	]

	focus gt-url
	view/new/title gt-face "Goto a URL"
]

